"use strict";
const decorateLanguageService_1 = require("@volar/typescript/lib/node/decorateLanguageService");
const decorateLanguageServiceHost_1 = require("@volar/typescript/lib/node/decorateLanguageServiceHost");
const language_core_1 = require("@vue/language-core");
const utils_1 = require("./lib/utils");
const vue = require("@vue/language-core");
const server_1 = require("./lib/server");
const componentInfos_1 = require("./lib/requests/componentInfos");
const shared_1 = require("@vue/shared");
const windowsPathReg = /\\/g;
const externalFiles = new WeakMap();
const projectExternalFileExtensions = new WeakMap();
const decoratedLanguageServices = new WeakSet();
const decoratedLanguageServiceHosts = new WeakSet();
function createLanguageServicePlugin() {
    return modules => {
        const { typescript: ts } = modules;
        const pluginModule = {
            create(info) {
                if (!decoratedLanguageServices.has(info.languageService)
                    && !decoratedLanguageServiceHosts.has(info.languageServiceHost)) {
                    decoratedLanguageServices.add(info.languageService);
                    decoratedLanguageServiceHosts.add(info.languageServiceHost);
                    const vueOptions = vue.resolveVueCompilerOptions(getVueCompilerOptions());
                    const languagePlugin = vue.createVueLanguagePlugin(ts, id => id, info.languageServiceHost.getCompilationSettings(), vueOptions);
                    const extensions = languagePlugin.typescript?.extraFileExtensions.map(ext => '.' + ext.extension) ?? [];
                    const getScriptSnapshot = info.languageServiceHost.getScriptSnapshot.bind(info.languageServiceHost);
                    const files = (0, language_core_1.createFileRegistry)([languagePlugin], ts.sys.useCaseSensitiveFileNames, fileName => {
                        const snapshot = getScriptSnapshot(fileName);
                        if (snapshot) {
                            files.set(fileName, (0, language_core_1.resolveCommonLanguageId)(fileName), snapshot);
                        }
                        else {
                            files.delete(fileName);
                        }
                    });
                    projectExternalFileExtensions.set(info.project, extensions);
                    utils_1.projects.set(info.project, {
                        info,
                        files,
                        ts,
                        vueOptions,
                    });
                    (0, decorateLanguageService_1.decorateLanguageService)(files, info.languageService);
                    (0, decorateLanguageServiceHost_1.decorateLanguageServiceHost)(files, info.languageServiceHost, ts);
                    (0, server_1.startNamedPipeServer)(info.project.projectKind, info.project.getCurrentDirectory());
                    const getCompletionsAtPosition = info.languageService.getCompletionsAtPosition;
                    const getCompletionEntryDetails = info.languageService.getCompletionEntryDetails;
                    const getCodeFixesAtPosition = info.languageService.getCodeFixesAtPosition;
                    const getEncodedSemanticClassifications = info.languageService.getEncodedSemanticClassifications;
                    info.languageService.getCompletionsAtPosition = (fileName, position, options) => {
                        const result = getCompletionsAtPosition(fileName, position, options);
                        if (result) {
                            // filter __VLS_
                            result.entries = result.entries.filter(entry => entry.name.indexOf('__VLS_') === -1
                                && (!entry.labelDetails?.description || entry.labelDetails.description.indexOf('__VLS_') === -1));
                            // modify label
                            for (const item of result.entries) {
                                if (item.source) {
                                    const originalName = item.name;
                                    for (const ext of vueOptions.extensions) {
                                        const suffix = (0, shared_1.capitalize)(ext.substring('.'.length)); // .vue -> Vue
                                        if (item.source.endsWith(ext) && item.name.endsWith(suffix)) {
                                            item.name = item.name.slice(0, -suffix.length);
                                            if (item.insertText) {
                                                // #2286
                                                item.insertText = item.insertText.replace(`${suffix}$1`, '$1');
                                            }
                                            if (item.data) {
                                                // @ts-expect-error
                                                item.data.__isComponentAutoImport = {
                                                    ext,
                                                    suffix,
                                                    originalName,
                                                    newName: item.insertText,
                                                };
                                            }
                                            break;
                                        }
                                    }
                                }
                            }
                        }
                        return result;
                    };
                    info.languageService.getCompletionEntryDetails = (...args) => {
                        const details = getCompletionEntryDetails(...args);
                        // modify import statement
                        // @ts-expect-error
                        if (args[6]?.__isComponentAutoImport) {
                            // @ts-expect-error
                            const { ext, suffix, originalName, newName } = args[6]?.__isComponentAutoImport;
                            for (const codeAction of details?.codeActions ?? []) {
                                for (const change of codeAction.changes) {
                                    for (const textChange of change.textChanges) {
                                        textChange.newText = textChange.newText.replace('import ' + originalName + ' from ', 'import ' + newName + ' from ');
                                    }
                                }
                            }
                        }
                        return details;
                    };
                    info.languageService.getCodeFixesAtPosition = (...args) => {
                        let result = getCodeFixesAtPosition(...args);
                        // filter __VLS_
                        result = result.filter(entry => entry.description.indexOf('__VLS_') === -1);
                        return result;
                    };
                    info.languageService.getEncodedSemanticClassifications = (fileName, span, format) => {
                        const result = getEncodedSemanticClassifications(fileName, span, format);
                        const file = files.get(fileName);
                        if (file?.generated?.code instanceof vue.VueGeneratedCode
                            && file.generated.code.sfc.template) {
                            const validComponentNames = (0, componentInfos_1._getComponentNames)(ts, info.languageService, file.generated.code, vueOptions);
                            const components = new Set([
                                ...validComponentNames,
                                ...validComponentNames.map(vue.hyphenateTag),
                            ]);
                            const { template } = file.generated.code.sfc;
                            const spanTemplateRange = [
                                span.start - template.startTagEnd,
                                span.start + span.length - template.startTagEnd,
                            ];
                            template.ast?.children.forEach(function visit(node) {
                                if (node.loc.end.offset <= spanTemplateRange[0] || node.loc.start.offset >= spanTemplateRange[1]) {
                                    return;
                                }
                                if (node.type === 1) {
                                    if (components.has(node.tag)) {
                                        result.spans.push(node.loc.start.offset + node.loc.source.indexOf(node.tag) + template.startTagEnd, node.tag.length, 256);
                                        if (template.lang === 'html' && !node.isSelfClosing) {
                                            result.spans.push(node.loc.start.offset + node.loc.source.lastIndexOf(node.tag) + template.startTagEnd, node.tag.length, 256);
                                        }
                                    }
                                    for (const child of node.children) {
                                        visit(child);
                                    }
                                }
                                else if (node.type === 9) {
                                    for (const branch of node.branches) {
                                        for (const child of branch.children) {
                                            visit(child);
                                        }
                                    }
                                }
                                else if (node.type === 11) {
                                    for (const child of node.children) {
                                        visit(child);
                                    }
                                }
                            });
                        }
                        return result;
                    };
                }
                return info.languageService;
                function getVueCompilerOptions() {
                    if (info.project.projectKind === ts.server.ProjectKind.Configured) {
                        const tsconfig = info.project.getProjectName();
                        return vue.createParsedCommandLine(ts, ts.sys, tsconfig.replace(windowsPathReg, '/')).vueOptions;
                    }
                    else {
                        return vue.createParsedCommandLineByJson(ts, ts.sys, info.languageServiceHost.getCurrentDirectory(), {}).vueOptions;
                    }
                }
            },
            getExternalFiles(project, updateLevel = 0) {
                if (updateLevel >= (1)
                    || !externalFiles.has(project)) {
                    const oldFiles = externalFiles.get(project);
                    const newFiles = (0, decorateLanguageServiceHost_1.searchExternalFiles)(ts, project, projectExternalFileExtensions.get(project));
                    externalFiles.set(project, newFiles);
                    if (oldFiles && !arrayItemsEqual(oldFiles, newFiles)) {
                        project.refreshDiagnostics();
                    }
                }
                return externalFiles.get(project);
            },
        };
        return pluginModule;
    };
}
function arrayItemsEqual(a, b) {
    if (a.length !== b.length) {
        return false;
    }
    const set = new Set(a);
    for (const file of b) {
        if (!set.has(file)) {
            return false;
        }
    }
    return true;
}
module.exports = createLanguageServicePlugin();
//# sourceMappingURL=index.js.map